@@ -1,4 +1,4 @@
-# Copyright 2020 - Omar Sandoval
+# Copyright (c) Facebook, Inc. and its affiliates.
 # SPDX-License-Identifier: GPL-3.0+
 
 import ast
@@ -266,6 +266,9 @@ class Formatter:
         node = resolved.node
 
         if rst:
+            if node.docstring is None:
+                want_rtype = False
+
             params_need_type = set()
             params_have_type = set()
             for line in lines:
