@@ -58,6 +58,7 @@ dwfl_frame_register (Dwfl_Frame *state, unsigned regno, Dwarf_Addr *val)
     *val = state->regs[regno];
   return true;
 }
+INTDEF (dwfl_frame_register)
 
 bool
 internal_function
@@ -698,6 +699,95 @@ readfunc (Dwarf_Addr addr, Dwarf_Word *datap, void *arg)
 					  process->callbacks_arg);
 }
 
+/* Internal version of dwfl_frame_module when we already know the PC.  */
+static Dwfl_Module *
+dwfl_frame_module_pc (Dwfl_Frame *state, Dwarf_Addr pc)
+{
+  if (state->mod != NULL)
+    return state->mod;
+  if (state->moderr == DWFL_E_NOERROR)
+    {
+      state->mod = INTUSE(dwfl_addrmodule) (state->thread->process->dwfl, pc);
+      if (state->mod != NULL)
+	return state->mod;
+      state->moderr = DWFL_E_NO_DWARF;
+    }
+  __libdwfl_seterrno (state->moderr);
+  return NULL;
+}
+
+/* Internal version of dwfl_frame_dwarf_frame when we already know the PC.  */
+static Dwarf_Frame *
+dwfl_frame_dwarf_frame_pc (Dwfl_Frame *state, Dwarf_Addr pc, Dwarf_Addr *bias)
+{
+  if (state->frame == NULL)
+    {
+      if (state->frameerr == DWFL_E_NOERROR)
+	{
+	  Dwfl_Module *mod = dwfl_frame_module_pc (state, pc);
+	  if (mod == NULL)
+	    {
+	      state->frameerr = state->moderr;
+	      /* errno is already set.  */
+	      return NULL;
+	    }
+	  Dwarf_CFI *cfi = INTUSE(dwfl_module_eh_cfi) (mod, &state->bias);
+	  if (cfi
+	      && INTUSE(dwarf_cfi_addrframe) (cfi, pc - state->bias,
+					      &state->frame) == 0)
+	    goto out;
+	  cfi = INTUSE(dwfl_module_dwarf_cfi) (mod, &state->bias);
+	  if (cfi
+	      && INTUSE(dwarf_cfi_addrframe) (cfi, pc - state->bias,
+					      &state->frame) == 0)
+	    goto out;
+	  state->frameerr = DWFL_E_NO_DWARF;
+	}
+      __libdwfl_seterrno (state->frameerr);
+      return NULL;
+    }
+
+out:
+  *bias = state->bias;
+  return state->frame;
+}
+
+Dwfl_Module *
+dwfl_frame_module (Dwfl_Frame *state)
+{
+  if (state->mod != NULL)
+    return state->mod;
+  if (state->moderr == DWFL_E_NOERROR)
+    {
+      Dwarf_Addr pc;
+      bool isactivation;
+      INTUSE(dwfl_frame_pc) (state, &pc, &isactivation);
+      if (! isactivation)
+	pc--;
+      return dwfl_frame_module_pc (state, pc);
+    }
+  __libdwfl_seterrno (state->moderr);
+  return NULL;
+}
+
+Dwarf_Frame *
+dwfl_frame_dwarf_frame (Dwfl_Frame *state, Dwarf_Addr *bias)
+{
+  if (state->frame != NULL)
+    return state->frame;
+  if (state->frameerr == DWFL_E_NOERROR)
+    {
+      Dwarf_Addr pc;
+      bool isactivation;
+      INTUSE(dwfl_frame_pc) (state, &pc, &isactivation);
+      if (! isactivation)
+	pc--;
+      return dwfl_frame_dwarf_frame_pc (state, pc, bias);
+    }
+  __libdwfl_seterrno (state->frameerr);
+  return NULL;
+}
+
 void
 internal_function
 __libdwfl_frame_unwind (Dwfl_Frame *state)
@@ -714,7 +804,7 @@ __libdwfl_frame_unwind (Dwfl_Frame *state)
   if (! state->initial_frame && ! state->signal_frame)
     pc--;
   Dwarf_Addr bias;
-  Dwarf_Frame *frame = INTUSE(dwfl_frame_dwarf_frame) (state, &bias);
+  Dwarf_Frame *frame = dwfl_frame_dwarf_frame_pc (state, pc, &bias);
   if (frame != NULL)
     {
       if (new_unwound (state) == NULL)
@@ -748,61 +838,6 @@ __libdwfl_frame_unwind (Dwfl_Frame *state)
   state->unwound->signal_frame = signal_frame;
 }
 
-Dwfl_Module *
-dwfl_frame_module (Dwfl_Frame *state)
-{
-  if (state->mod != NULL)
-    return state->mod;
-  if (state->moderr == DWFL_E_NOERROR)
-    {
-      Dwarf_Addr pc;
-      INTUSE(dwfl_frame_pc) (state, &pc, NULL);
-      state->mod = INTUSE(dwfl_addrmodule) (state->thread->process->dwfl, pc);
-      if (state->mod != NULL)
-	return state->mod;
-      state->moderr = DWFL_E_NO_DWARF;
-    }
-  __libdwfl_seterrno (state->moderr);
-  return NULL;
-}
-
-Dwarf_Frame *
-dwfl_frame_dwarf_frame (Dwfl_Frame *state, Dwarf_Addr *bias)
-{
-  if (state->frame == NULL)
-    {
-      if (state->frameerr == DWFL_E_NOERROR)
-	{
-	  Dwfl_Module *mod = INTUSE(dwfl_frame_module) (state);
-	  if (mod == NULL)
-	    {
-	      state->frameerr = state->moderr;
-	      return NULL;
-	    }
-	  Dwarf_Addr pc;
-	  INTUSE(dwfl_frame_pc) (state, &pc, NULL);
-	  Dwarf_CFI *cfi = INTUSE(dwfl_module_eh_cfi) (state->mod,
-						       &state->bias);
-	  if (cfi
-	      && INTUSE(dwarf_cfi_addrframe) (cfi, pc - state->bias,
-					      &state->frame) == 0)
-	    goto out;
-	  cfi = INTUSE(dwfl_module_dwarf_cfi) (state->mod, &state->bias);
-	  if (cfi
-	      && INTUSE(dwarf_cfi_addrframe) (cfi, pc - state->bias,
-					      &state->frame) == 0)
-	    goto out;
-	  state->frameerr = DWFL_E_NO_DWARF;
-	}
-      __libdwfl_seterrno (state->frameerr);
-      return NULL;
-    }
-
-out:
-  *bias = state->bias;
-  return state->frame;
-}
-
 bool
 dwfl_frame_eval_expr (Dwfl_Frame *state, const Dwarf_Op *ops, size_t nops,
 		      Dwarf_Addr *result)
