@@ -1,4 +1,4 @@
-// Copyright 2018-2020 - Omar Sandoval
+// Copyright (c) Facebook, Inc. and its affiliates.
 // SPDX-License-Identifier: GPL-3.0+
 
 #include <ctype.h>
@@ -123,7 +123,7 @@ c_append_tagged_name(struct drgn_qualified_type qualified_type, size_t indent,
 		keyword = "enum";
 		break;
 	default:
-		DRGN_UNREACHABLE();
+		UNREACHABLE();
 	}
 
 	if (!append_tabs(indent, sb))
@@ -352,7 +352,7 @@ c_declare_variable(struct drgn_qualified_type qualified_type,
 	case DRGN_TYPE_FUNCTION:
 		return c_declare_function(qualified_type, name, indent, sb);
 	}
-	DRGN_UNREACHABLE();
+	UNREACHABLE();
 }
 
 static struct drgn_error *
@@ -507,7 +507,7 @@ c_define_type(struct drgn_qualified_type qualified_type, size_t indent,
 		return drgn_error_create(DRGN_ERROR_INVALID_ARGUMENT,
 					 "function type cannot be formatted");
 	}
-	DRGN_UNREACHABLE();
+	UNREACHABLE();
 }
 
 static struct drgn_error *
@@ -725,7 +725,7 @@ c_format_int_object(const struct drgn_object *obj,
 		return NULL;
 	}
 	default:
-		DRGN_UNREACHABLE();
+		UNREACHABLE();
 	}
 }
 
@@ -1133,7 +1133,7 @@ c_format_compound_object(const struct drgn_object *obj,
 			keyword = "class";
 			break;
 		default:
-			DRGN_UNREACHABLE();
+			UNREACHABLE();
 		}
 		return drgn_error_format(DRGN_ERROR_TYPE,
 					 "cannot format incomplete %s object",
@@ -1599,7 +1599,7 @@ c_format_object_impl(const struct drgn_object *obj, size_t indent,
 	case DRGN_TYPE_FUNCTION:
 		return c_format_function_object(obj, sb);
 	default:
-		DRGN_UNREACHABLE();
+		UNREACHABLE();
 	}
 }
 
@@ -1682,15 +1682,12 @@ static const char *token_spelling[] = {
 
 DEFINE_HASH_MAP(c_keyword_map, struct string, int, string_hash, string_eq);
 
-static struct c_keyword_map c_keywords;
+static struct c_keyword_map c_keywords = HASH_TABLE_INIT;
 
 __attribute__((constructor(101)))
 static void c_keywords_init(void)
 {
-	int i;
-
-	c_keyword_map_init(&c_keywords);
-	for (i = MIN_KEYWORD_TOKEN; i <= MAX_KEYWORD_TOKEN; i++) {
+	for (int i = MIN_KEYWORD_TOKEN; i <= MAX_KEYWORD_TOKEN; i++) {
 		struct c_keyword_map_entry entry = {
 			.key = {
 				.str = token_spelling[i],
@@ -1698,7 +1695,6 @@ static void c_keywords_init(void)
 			},
 			.value = i,
 		};
-
 		if (c_keyword_map_insert(&c_keywords, &entry, NULL) != 1)
 			abort();
 	}
@@ -2681,7 +2677,7 @@ struct drgn_error *c_bit_offset(struct drgn_program *prog,
 			}
 			break;
 		default:
-			DRGN_UNREACHABLE();
+			UNREACHABLE();
 		}
 		state = token.kind;
 	}
@@ -2704,10 +2700,7 @@ struct drgn_error *c_integer_literal(struct drgn_object *res, uint64_t uvalue)
 	struct drgn_qualified_type qualified_type;
 	size_t i;
 
-	_Static_assert(sizeof(unsigned long long) == 8,
-		       "unsigned long long is not 64 bits");
-	bits = uvalue ? 64 - __builtin_clzll(uvalue) : 0;
-
+	bits = fls(uvalue);
 	qualified_type.qualifiers = 0;
 	for (i = 0; i < ARRAY_SIZE(types); i++) {
 		err = drgn_type_index_find_primitive(&res->prog->tindex,
@@ -2933,7 +2926,7 @@ c_corresponding_unsigned_type(struct drgn_type_index *tindex,
 						      DRGN_C_TYPE_UNSIGNED_LONG_LONG,
 						      ret);
 	default:
-		DRGN_UNREACHABLE();
+		UNREACHABLE();
 	}
 }
 
