@@ -1,4 +1,4 @@
-// Copyright 2018-2020 - Omar Sandoval
+// Copyright (c) Facebook, Inc. and its affiliates.
 // SPDX-License-Identifier: GPL-3.0+
 
 #include <dirent.h>
@@ -14,10 +14,20 @@
 
 #include "internal.h"
 #include "dwarf_index.h"
+#include "helpers.h"
 #include "linux_kernel.h"
 #include "program.h"
 #include "read.h"
 
+struct drgn_error *read_memory_via_pgtable(void *buf, uint64_t address,
+					   size_t count, uint64_t offset,
+					   void *arg, bool physical)
+{
+	struct drgn_program *prog = arg;
+	return linux_helper_read_vm(prog, prog->vmcoreinfo.swapper_pg_dir,
+				    address, buf, count);
+}
+
 static inline bool linematch(const char **line, const char *prefix)
 {
 	size_t len = strlen(prefix);
@@ -83,6 +93,11 @@ struct drgn_error *parse_vmcoreinfo(const char *desc, size_t descsz,
 					  &ret->kaslr_offset);
 			if (err)
 				return err;
+		} else if (linematch(&line, "SYMBOL(swapper_pg_dir)=")) {
+			err = line_to_u64(line, newline, 16,
+					  &ret->swapper_pg_dir);
+			if (err)
+				return err;
 		} else if (linematch(&line, "NUMBER(pgtable_l5_enabled)=")) {
 			uint64_t tmp;
 
@@ -101,12 +116,16 @@ struct drgn_error *parse_vmcoreinfo(const char *desc, size_t descsz,
 		return drgn_error_create(DRGN_ERROR_OTHER,
 					 "VMCOREINFO does not contain valid PAGESIZE");
 	}
+	if (!ret->swapper_pg_dir) {
+		return drgn_error_create(DRGN_ERROR_OTHER,
+					 "VMCOREINFO does not contain valid swapper_pg_dir");
+	}
 	/* KERNELOFFSET and pgtable_l5_enabled are optional. */
 	return NULL;
 }
 
-static struct drgn_error *proc_kallsyms_symbol_addr(const char *name,
-						    unsigned long *ret)
+struct drgn_error *proc_kallsyms_symbol_addr(const char *name,
+					     unsigned long *ret)
 {
 	struct drgn_error *err;
 	FILE *file;
@@ -126,9 +145,7 @@ static struct drgn_error *proc_kallsyms_symbol_addr(const char *name,
 				err = drgn_error_create_os("getline", errno,
 							   "/proc/kallsyms");
 			} else {
-				err = drgn_error_format(DRGN_ERROR_OTHER,
-							"could not find %s symbol in /proc/kallsyms",
-							name);
+				err = &drgn_not_found;
 			}
 			break;
 		}
@@ -163,20 +180,16 @@ invalid:
 
 /*
  * Before Linux kernel commit 23c85094fe18 ("proc/kcore: add vmcoreinfo note to
- * /proc/kcore") (in v4.19), /proc/kcore didn't have a VMCOREINFO note, so we
- * have to get it by other means. Since Linux kernel commit 464920104bf7
- * ("/proc/kcore: update physical address for kcore ram and text") (in v4.11),
+ * /proc/kcore") (in v4.19), /proc/kcore didn't have a VMCOREINFO note. Instead,
  * we can read from the physical address of the vmcoreinfo note exported in
- * sysfs. Before that, p_paddr in /proc/kcore is always zero, but we can read
- * from the virtual address in /proc/kallsyms.
+ * sysfs.
  */
 struct drgn_error *read_vmcoreinfo_fallback(struct drgn_memory_reader *reader,
-					    bool have_non_zero_phys_addr,
 					    struct vmcoreinfo *ret)
 {
 	struct drgn_error *err;
 	FILE *file;
-	unsigned long address;
+	uint64_t address;
 	size_t size;
 	char *buf;
 	Elf64_Nhdr *nhdr;
@@ -186,32 +199,18 @@ struct drgn_error *read_vmcoreinfo_fallback(struct drgn_memory_reader *reader,
 		return drgn_error_create_os("fopen", errno,
 					    "/sys/kernel/vmcoreinfo");
 	}
-	if (fscanf(file, "%lx %zx", &address, &size) != 2) {
+	if (fscanf(file, "%" SCNx64 "%zx", &address, &size) != 2) {
 		fclose(file);
 		return drgn_error_create(DRGN_ERROR_OTHER,
 					 "could not parse /sys/kernel/vmcoreinfo");
 	}
 	fclose(file);
 
-	if (!have_non_zero_phys_addr) {
-		/*
-		 * Since Linux kernel commit 203e9e41219b ("kexec: move
-		 * vmcoreinfo out of the kernel's .bss section") (in v4.13),
-		 * vmcoreinfo_note is a pointer; before that, it is an array. We
-		 * only do this for kernels before v4.11, so we can assume that
-		 * it's an array.
-		 */
-		err = proc_kallsyms_symbol_addr("vmcoreinfo_note", &address);
-		if (err)
-			return err;
-	}
-
 	buf = malloc(size);
 	if (!buf)
 		return &drgn_enomem;
 
-	err = drgn_memory_reader_read(reader, buf, address, size,
-				      have_non_zero_phys_addr);
+	err = drgn_memory_reader_read(reader, buf, address, size, true);
 	if (err)
 		goto out;
 
@@ -235,6 +234,33 @@ out:
 	return err;
 }
 
+struct drgn_error *linux_kernel_get_thread_size(struct drgn_program *prog,
+						uint64_t *ret)
+{
+	struct drgn_error *err;
+	struct drgn_qualified_type thread_union_type;
+	struct drgn_member_info stack_member;
+
+	if (!prog->thread_size) {
+		err = drgn_program_find_type(prog, "union thread_union", NULL,
+					     &thread_union_type);
+		if (err)
+			return err;
+		err = drgn_program_member_info(prog, thread_union_type.type,
+					       "stack", &stack_member);
+		if (err)
+			return err;
+		err = drgn_type_sizeof(stack_member.qualified_type.type,
+				       &prog->thread_size);
+		if (err) {
+			prog->thread_size = 0;
+			return err;
+		}
+	}
+	*ret = prog->thread_size;
+	return NULL;
+}
+
 struct drgn_error *linux_kernel_object_find(const char *name, size_t name_len,
 					    const char *filename,
 					    enum drgn_find_object_flags flags,
@@ -297,6 +323,20 @@ struct drgn_error *linux_kernel_object_find(const char *name, size_t name_len,
 			return drgn_object_set_unsigned(ret, qualified_type,
 							~(prog->vmcoreinfo.page_size - 1),
 							0);
+		} else if (name_len == strlen("THREAD_SIZE") &&
+			   memcmp(name, "THREAD_SIZE", name_len) == 0) {
+			uint64_t thread_size;
+
+			err = linux_kernel_get_thread_size(prog, &thread_size);
+			if (err)
+				return err;
+			err = drgn_type_index_find_primitive(&prog->tindex,
+							     DRGN_C_TYPE_UNSIGNED_LONG,
+							     &qualified_type.type);
+			if (err)
+				return err;
+			return drgn_object_set_unsigned(ret, qualified_type,
+							thread_size, 0);
 		} else if (name_len == strlen("UTS_RELEASE") &&
 			   memcmp(name, "UTS_RELEASE", name_len) == 0) {
 			size_t len;
@@ -780,15 +820,19 @@ out:
 
 static const char *kmod_index_find(struct kmod_index *index, const char *key)
 {
+	static const uint32_t INDEX_NODE_MASK = UINT32_C(0x0fffffff);
+	static const uint32_t INDEX_NODE_CHILDS = UINT32_C(0x20000000);
+	static const uint32_t INDEX_NODE_VALUES = UINT32_C(0x40000000);
+	static const uint32_t INDEX_NODE_PREFIX = UINT32_C(0x80000000);
 	const char *ptr = index->ptr + 8;
 	uint32_t offset;
 
 	for (;;) {
 		if (!read_be32(&ptr, index->end, &offset))
 			return NULL;
-		ptr = index->ptr + (offset & 0x0fffffffU);
+		ptr = index->ptr + (offset & INDEX_NODE_MASK);
 
-		if (offset & 0x80000000U) {
+		if (offset & INDEX_NODE_PREFIX) {
 			const char *prefix;
 			size_t prefix_len;
 
@@ -800,7 +844,7 @@ static const char *kmod_index_find(struct kmod_index *index, const char *key)
 			key += prefix_len;
 		}
 
-		if (offset & 0x20000000U) {
+		if (offset & INDEX_NODE_CHILDS) {
 			uint8_t first, last;
 
 			if (!read_u8(&ptr, index->end, &first) ||
@@ -824,7 +868,7 @@ static const char *kmod_index_find(struct kmod_index *index, const char *key)
 			break;
 		}
 	}
-	if (!(offset & 0x40000000U))
+	if (!(offset & INDEX_NODE_VALUES))
 		return NULL;
 	return ptr;
 }
@@ -1002,23 +1046,16 @@ cache_kernel_module_sections(struct kernel_module_iterator *kmod_it, Elf *elf,
 			     uint64_t *start_ret, uint64_t *end_ret)
 {
 	struct drgn_error *err;
-	uint64_t start = UINT64_MAX, end = 0;
-	size_t shstrndx;
-	Elf_Scn *scn = NULL;
-	struct elf_scn_name_map scn_map;
-	struct kernel_module_section_iterator section_it;
-	const char *name;
-	uint64_t address;
 
+	size_t shstrndx;
 	if (elf_getshdrstrndx(elf, &shstrndx))
 		return drgn_error_libelf();
 
-	elf_scn_name_map_init(&scn_map);
+	struct elf_scn_name_map scn_map = HASH_TABLE_INIT;
+	Elf_Scn *scn = NULL;
 	while ((scn = elf_nextscn(elf, scn))) {
-		GElf_Shdr *shdr, shdr_mem;
-		struct elf_scn_name_map_entry entry;
-
-		shdr = gelf_getshdr(scn, &shdr_mem);
+		GElf_Shdr shdr_mem;
+		GElf_Shdr *shdr = gelf_getshdr(scn, &shdr_mem);
 		if (!shdr) {
 			err = drgn_error_libelf();
 			goto out_scn_map;
@@ -1027,7 +1064,10 @@ cache_kernel_module_sections(struct kernel_module_iterator *kmod_it, Elf *elf,
 		if (!(shdr->sh_flags & SHF_ALLOC))
 			continue;
 
-		entry.key = elf_strptr(elf, shstrndx, shdr->sh_name);
+		struct elf_scn_name_map_entry entry = {
+			.key = elf_strptr(elf, shstrndx, shdr->sh_name),
+			.value = scn,
+		};
 		/*
 		 * .init sections are freed once the module is initialized, but
 		 * they remain in the section list. Ignore them so we don't get
@@ -1035,7 +1075,6 @@ cache_kernel_module_sections(struct kernel_module_iterator *kmod_it, Elf *elf,
 		 */
 		if (!entry.key || strstartswith(entry.key, ".init"))
 			continue;
-		entry.value = scn;
 
 		if (elf_scn_name_map_insert(&scn_map, &entry, NULL) == -1) {
 			err = &drgn_enomem;
@@ -1043,19 +1082,21 @@ cache_kernel_module_sections(struct kernel_module_iterator *kmod_it, Elf *elf,
 		}
 	}
 
+	uint64_t start = UINT64_MAX, end = 0;
+	struct kernel_module_section_iterator section_it;
 	err = kernel_module_section_iterator_init(&section_it, kmod_it);
 	if (err)
 		goto out_scn_map;
+	const char *name;
+	uint64_t address;
 	while (!(err = kernel_module_section_iterator_next(&section_it, &name,
 							   &address))) {
-		struct elf_scn_name_map_iterator it;
-
-		it = elf_scn_name_map_search(&scn_map, &name);
+		struct elf_scn_name_map_iterator it =
+			elf_scn_name_map_search(&scn_map, &name);
 		if (it.entry) {
-			GElf_Shdr *shdr, shdr_mem;
-			uint64_t section_end;
-
-			shdr = gelf_getshdr(it.entry->value, &shdr_mem);
+			GElf_Shdr shdr_mem;
+			GElf_Shdr *shdr = gelf_getshdr(it.entry->value,
+						       &shdr_mem);
 			if (!shdr) {
 				err = drgn_error_libelf();
 				break;
@@ -1065,6 +1106,7 @@ cache_kernel_module_sections(struct kernel_module_iterator *kmod_it, Elf *elf,
 				err = drgn_error_libelf();
 				break;
 			}
+			uint64_t section_end;
 			if (__builtin_add_overflow(address, shdr->sh_size,
 						   &section_end))
 				section_end = UINT64_MAX;
@@ -1300,11 +1342,6 @@ report_kernel_modules(struct drgn_program *prog,
 		      bool vmlinux_is_pending)
 {
 	struct drgn_error *err;
-	struct kernel_module_table kmod_table;
-	struct depmod_index depmod;
-	size_t module_name_offset = 0;
-	size_t i;
-	struct kernel_module_table_iterator it;
 
 	if (!num_kmods && !report_default)
 		return NULL;
@@ -1323,10 +1360,10 @@ report_kernel_modules(struct drgn_program *prog,
 			return err;
 	}
 
+	size_t module_name_offset = 0;
 	if (need_module_definition) {
 		struct drgn_qualified_type module_type;
 		struct drgn_member_info name_member;
-
 		err = drgn_program_find_type(prog, "struct module", NULL,
 					     &module_type);
 		if (!err) {
@@ -1342,12 +1379,12 @@ report_kernel_modules(struct drgn_program *prog,
 		module_name_offset = name_member.bit_offset / 8;
 	}
 
-	kernel_module_table_init(&kmod_table);
+	struct kernel_module_table kmod_table = HASH_TABLE_INIT;
+	struct depmod_index depmod;
+	struct kernel_module_table_iterator it;
 	depmod.modules_dep.ptr = NULL;
-	for (i = 0; i < num_kmods; i++) {
+	for (size_t i = 0; i < num_kmods; i++) {
 		struct kernel_module_file *kmod = &kmods[i];
-		struct hash_pair hp;
-
 		if (!kmod->name) {
 			err = get_kernel_module_name_from_this_module(kmod->this_module_scn,
 								      module_name_offset,
@@ -1371,7 +1408,7 @@ report_kernel_modules(struct drgn_program *prog,
 			}
 		}
 
-		hp = kernel_module_table_hash(&kmod->name);
+		struct hash_pair hp = kernel_module_table_hash(&kmod->name);
 		it = kernel_module_table_search_hashed(&kmod_table, &kmod->name,
 						       hp);
 		if (it.entry) {
@@ -1397,7 +1434,6 @@ report_kernel_modules(struct drgn_program *prog,
 	/* Anything left over was not loaded. */
 	for (it = kernel_module_table_first(&kmod_table); it.entry; ) {
 		struct kernel_module_file *kmod = *it.entry;
-
 		it = kernel_module_table_delete_iterator(&kmod_table, it);
 		do {
 			err = drgn_dwarf_index_report_elf(dindex, kmod->path,
