@@ -1,4 +1,4 @@
-// Copyright 2018-2019 - Omar Sandoval
+// Copyright (c) Facebook, Inc. and its affiliates.
 // SPDX-License-Identifier: GPL-3.0+
 
 #include <inttypes.h>
@@ -104,7 +104,7 @@ drgn_object_set_common(struct drgn_qualified_type qualified_type,
 	return drgn_object_type_kind_and_size(type_ret, kind_ret, bit_size_ret);
 }
 
-static struct drgn_error *
+struct drgn_error *
 drgn_object_set_signed_internal(struct drgn_object *res,
 				const struct drgn_object_type *type,
 				uint64_t bit_size, int64_t svalue)
@@ -140,7 +140,7 @@ drgn_object_set_signed(struct drgn_object *res,
 	return drgn_object_set_signed_internal(res, &type, bit_size, svalue);
 }
 
-static struct drgn_error *
+struct drgn_error *
 drgn_object_set_unsigned_internal(struct drgn_object *res,
 				  const struct drgn_object_type *type,
 				  uint64_t bit_size, uint64_t uvalue)
@@ -244,7 +244,7 @@ struct drgn_error *sanity_check_object(enum drgn_object_kind kind,
 	}
 }
 
-static void drgn_value_deserialize(union drgn_value *value, const char *buf,
+static void drgn_value_deserialize(union drgn_value *value, const void *buf,
 				   uint8_t bit_offset,
 				   enum drgn_object_kind kind,
 				   uint64_t bit_size, bool little_endian)
@@ -268,7 +268,7 @@ static void drgn_value_deserialize(union drgn_value *value, const char *buf,
 		value->fvalue = bit_size == 32 ? tmp.fvalue32 : tmp.fvalue64;
 		break;
 	default:
-		DRGN_UNREACHABLE();
+		UNREACHABLE();
 	}
 }
 
@@ -296,11 +296,11 @@ drgn_byte_order_to_little_endian(struct drgn_program *prog,
 	}
 }
 
-static struct drgn_error *
+struct drgn_error *
 drgn_object_set_buffer_internal(struct drgn_object *res,
 				const struct drgn_object_type *type,
 				enum drgn_object_kind kind, uint64_t bit_size,
-				const char *buf, uint8_t bit_offset,
+				const void *buf, uint8_t bit_offset,
 				bool little_endian)
 {
 	struct drgn_error *err;
@@ -352,7 +352,7 @@ drgn_object_set_buffer_internal(struct drgn_object *res,
 LIBDRGN_PUBLIC struct drgn_error *
 drgn_object_set_buffer(struct drgn_object *res,
 		       struct drgn_qualified_type qualified_type,
-		       const char *buf, uint8_t bit_offset,
+		       const void *buf, uint8_t bit_offset,
 		       uint64_t bit_field_size, enum drgn_byte_order byte_order)
 {
 	struct drgn_error *err;
@@ -565,11 +565,9 @@ drgn_object_dereference_offset(struct drgn_object *res,
 
 	/*
 	 * / and % truncate towards 0. Here, we want to truncate towards
-	 * negative infinity. As long as we have an arithmetic right shift, we
-	 * can accomplish that by replacing "/ 8" with ">> 3" and "% 8" with
-	 * "& 7".
+	 * negative infinity. We can accomplish that by replacing "/ 8" with an
+	 * arithmetic shift ">> 3" and "% 8" with "& 7".
 	 */
-	static_assert((-1 >> 1) == -1, "right shift is not arithmetic");
 	address += bit_offset >> 3;
 	bit_offset &= 7;
 	return drgn_object_set_reference(res, qualified_type, address,
@@ -2167,11 +2165,6 @@ struct drgn_error *drgn_op_rshift_impl(struct drgn_object *res,
 		err = drgn_object_convert_signed(lhs, bit_size, &svalue);
 		if (err)
 			return err;
-		/*
-		 * Right shift of a negative integer is implementation-defined.
-		 * GCC always uses an arithmetic shift.
-		 */
-		static_assert((-1 >> 1) == -1, "right shift is not arithmetic");
 		if (shift < bit_size)
 			svalue >>= shift;
 		else if (svalue >= 0)
@@ -2236,7 +2229,7 @@ INTEGER_BINARY_OP(xor, ^)
 	uint64_t _bit_size = (bit_size);					\
 	union {									\
 		int64_t svalue;							\
-		int64_t uvalue;							\
+		uint64_t uvalue;						\
 	} tmp;									\
 										\
 	_err = drgn_object_convert_signed((obj), _bit_size, &tmp.svalue);	\
