@@ -1,4 +1,4 @@
-// Copyright 2018-2019 - Omar Sandoval
+// Copyright (c) Facebook, Inc. and its affiliates.
 // SPDX-License-Identifier: GPL-3.0+
 
 /**
@@ -130,12 +130,36 @@ struct drgn_error *sanity_check_object(enum drgn_object_kind kind,
 				       uint64_t bit_size);
 
 /**
- * Convert a @ref drgn_byte_order to a boolean.
- *
- * @return @c true if the byte order is little endian, @c false if the byte
- * order is big endian.
+ * Like @ref drgn_object_set_signed() but @ref drgn_object_set_common() was
+ * already called.
  */
 struct drgn_error *
+drgn_object_set_signed_internal(struct drgn_object *res,
+				const struct drgn_object_type *type,
+				uint64_t bit_size, int64_t svalue);
+
+/**
+ * Like @ref drgn_object_set_unsigned() but @ref drgn_object_set_common() was
+ * already called.
+ */
+struct drgn_error *
+drgn_object_set_unsigned_internal(struct drgn_object *res,
+				  const struct drgn_object_type *type,
+				  uint64_t bit_size, uint64_t uvalue);
+
+/**
+ * Like @ref drgn_object_set_buffer() but @ref drgn_object_set_common() was
+ * already called.
+ */
+struct drgn_error *
+drgn_object_set_buffer_internal(struct drgn_object *res,
+				const struct drgn_object_type *type,
+				enum drgn_object_kind kind, uint64_t bit_size,
+				const void *buf, uint8_t bit_offset,
+				bool little_endian);
+
+/** Convert a @ref drgn_byte_order to a boolean. */
+struct drgn_error *
 drgn_byte_order_to_little_endian(struct drgn_program *prog,
 				 enum drgn_byte_order byte_order, bool *ret);
 
