@@ -1,4 +1,4 @@
-// Copyright 2018-2019 - Omar Sandoval
+// Copyright (c) Facebook, Inc. and its affiliates.
 // SPDX-License-Identifier: GPL-3.0+
 
 #include <byteswap.h>
@@ -28,6 +28,7 @@
 #include "type_index.h"
 #include "vector.h"
 
+DEFINE_VECTOR_FUNCTIONS(drgn_prstatus_vector)
 DEFINE_HASH_TABLE_FUNCTIONS(drgn_prstatus_map, hash_pair_int_type,
 			    hash_table_scalar_eq)
 
@@ -75,7 +76,6 @@ void drgn_program_init(struct drgn_program *prog,
 	drgn_type_index_init(&prog->tindex);
 	drgn_object_index_init(&prog->oindex);
 	prog->core_fd = -1;
-	drgn_prstatus_map_init(&prog->prstatus_cache);
 	if (platform)
 		drgn_program_set_platform(prog, platform);
 }
@@ -83,7 +83,13 @@ void drgn_program_init(struct drgn_program *prog,
 void drgn_program_deinit(struct drgn_program *prog)
 {
 	free(prog->task_state_chars);
-	drgn_prstatus_map_deinit(&prog->prstatus_cache);
+	if (prog->prstatus_cached) {
+		if (prog->flags & DRGN_PROGRAM_IS_LINUX_KERNEL)
+			drgn_prstatus_vector_deinit(&prog->prstatus_vector);
+		else
+			drgn_prstatus_map_deinit(&prog->prstatus_map);
+	}
+	free(prog->pgtable_it);
 
 	drgn_object_index_deinit(&prog->oindex);
 	drgn_type_index_deinit(&prog->tindex);
@@ -189,9 +195,8 @@ drgn_program_set_core_dump(struct drgn_program *prog, const char *path)
 	struct drgn_platform platform;
 	bool is_64_bit, is_kdump;
 	size_t phnum, i;
-	size_t num_file_segments;
-	bool have_non_zero_phys_addr = false;
-	struct drgn_memory_file_segment *current_file_segment;
+	size_t num_file_segments, j;
+	bool have_phys_addrs = false;
 	const char *vmcoreinfo_note = NULL;
 	size_t vmcoreinfo_size = 0;
 	bool have_nt_taskstruct = false, is_proc_kcore;
@@ -253,7 +258,7 @@ drgn_program_set_core_dump(struct drgn_program *prog, const char *path)
 
 		if (phdr->p_type == PT_LOAD) {
 			if (phdr->p_paddr)
-				have_non_zero_phys_addr = true;
+				have_phys_addrs = true;
 			num_file_segments++;
 		} else if (phdr->p_type == PT_NOTE) {
 			Elf_Data *data;
@@ -285,6 +290,12 @@ drgn_program_set_core_dump(struct drgn_program *prog, const char *path)
 						   nhdr.n_namesz) == 0) {
 					vmcoreinfo_note = desc;
 					vmcoreinfo_size = nhdr.n_descsz;
+					/*
+					 * This is either a vmcore or
+					 * /proc/kcore, so even a p_paddr of 0
+					 * may be valid.
+					 */
+					have_phys_addrs = true;
 				}
 			}
 		}
@@ -309,23 +320,10 @@ drgn_program_set_core_dump(struct drgn_program *prog, const char *path)
 
 	if (vmcoreinfo_note && !is_proc_kcore) {
 		char *env;
-		bool use_libkdumpfile;
 
-		/*
-		 * Use libkdumpfile for ELF vmcores if we were compiled with
-		 * libkdumpfile support unless specified otherwise.
-		 */
+		/* Use libkdumpfile for ELF vmcores if it was requested. */
 		env = getenv("DRGN_USE_LIBKDUMPFILE_FOR_ELF");
-		if (env) {
-			use_libkdumpfile = atoi(env);
-		} else {
-#ifdef WITH_LIBKDUMPFILE
-			use_libkdumpfile = true;
-#else
-			use_libkdumpfile = false;
-#endif
-		}
-		if (use_libkdumpfile) {
+		if (env && atoi(env)) {
 			err = drgn_program_set_kdump(prog);
 			if (err)
 				goto out_elf;
@@ -339,11 +337,23 @@ drgn_program_set_core_dump(struct drgn_program *prog, const char *path)
 		err = &drgn_enomem;
 		goto out_elf;
 	}
-	prog->num_file_segments = num_file_segments;
-	current_file_segment = prog->file_segments;
+
+	if (is_proc_kcore || vmcoreinfo_note) {
+		/*
+		 * Try to read any memory that isn't in the core dump via the
+		 * page table.
+		 */
+		err = drgn_program_add_memory_segment(prog, 0,
+						      is_64_bit ?
+						      UINT64_MAX : UINT32_MAX,
+						      read_memory_via_pgtable,
+						      prog, false);
+		if (err)
+			goto out_segments;
+	}
 
 	/* Second pass: add the segments. */
-	for (i = 0; i < phnum; i++) {
+	for (i = 0, j = 0; i < phnum && j < num_file_segments; i++) {
 		GElf_Phdr phdr_mem, *phdr;
 
 		phdr = gelf_getphdr(prog->core, i, &phdr_mem);
@@ -352,40 +362,78 @@ drgn_program_set_core_dump(struct drgn_program *prog, const char *path)
 			goto out_segments;
 		}
 
-		if (phdr->p_type == PT_LOAD) {
-			/*
-			 * If this happens, then the number of segments changed
-			 * since the first pass. That's probably impossible, but
-			 * skip it just in case.
-			 */
-			if (current_file_segment ==
-			    prog->file_segments + prog->num_file_segments)
-				continue;
-			current_file_segment->file_offset = phdr->p_offset;
-			current_file_segment->file_size = phdr->p_filesz;
-			current_file_segment->fd = prog->core_fd;
-			current_file_segment->eio_is_fault = false;
+		if (phdr->p_type != PT_LOAD)
+			continue;
+
+		prog->file_segments[j].file_offset = phdr->p_offset;
+		prog->file_segments[j].file_size = phdr->p_filesz;
+		prog->file_segments[j].fd = prog->core_fd;
+		prog->file_segments[j].eio_is_fault = false;
+		err = drgn_program_add_memory_segment(prog, phdr->p_vaddr,
+						      phdr->p_memsz,
+						      drgn_read_memory_file,
+						      &prog->file_segments[j],
+						      false);
+		if (err)
+			goto out_segments;
+		if (have_phys_addrs &&
+		    phdr->p_paddr != (is_64_bit ? UINT64_MAX : UINT32_MAX)) {
 			err = drgn_program_add_memory_segment(prog,
-							      phdr->p_vaddr,
+							      phdr->p_paddr,
 							      phdr->p_memsz,
 							      drgn_read_memory_file,
-							      current_file_segment,
-							      false);
+							      &prog->file_segments[j],
+							      true);
 			if (err)
 				goto out_segments;
-			if (have_non_zero_phys_addr &&
-			    phdr->p_paddr !=
-			    (is_64_bit ? UINT64_MAX : UINT32_MAX)) {
+		}
+		j++;
+	}
+	/*
+	 * Before Linux kernel commit 464920104bf7 ("/proc/kcore: update
+	 * physical address for kcore ram and text") (in v4.11), p_paddr in
+	 * /proc/kcore is always zero. If we know the address of the direct
+	 * mapping, we can still add physical segments. This needs to be a third
+	 * pass, as we may need to read virtual memory to determine the mapping.
+	 */
+	if (is_proc_kcore && !have_phys_addrs &&
+	    platform.arch->linux_kernel_live_direct_mapping_fallback) {
+		uint64_t direct_mapping, direct_mapping_size;
+
+		err = platform.arch->linux_kernel_live_direct_mapping_fallback(prog,
+									       &direct_mapping,
+									       &direct_mapping_size);
+		if (err)
+			goto out_segments;
+
+		for (i = 0, j = 0; i < phnum && j < num_file_segments; i++) {
+			GElf_Phdr phdr_mem, *phdr;
+
+			phdr = gelf_getphdr(prog->core, i, &phdr_mem);
+			if (!phdr) {
+				err = drgn_error_libelf();
+				goto out_segments;
+			}
+
+			if (phdr->p_type != PT_LOAD)
+				continue;
+
+			if (phdr->p_vaddr >= direct_mapping &&
+			    phdr->p_vaddr - direct_mapping + phdr->p_memsz <=
+			    direct_mapping_size) {
+				uint64_t phys_addr;
+
+				phys_addr = phdr->p_vaddr - direct_mapping;
 				err = drgn_program_add_memory_segment(prog,
-								      phdr->p_paddr,
+								      phys_addr,
 								      phdr->p_memsz,
 								      drgn_read_memory_file,
-								      current_file_segment,
+								      &prog->file_segments[j],
 								      true);
 				if (err)
 					goto out_segments;
 			}
-			current_file_segment++;
+			j++;
 		}
 	}
 	if (vmcoreinfo_note) {
@@ -398,7 +446,6 @@ drgn_program_set_core_dump(struct drgn_program *prog, const char *path)
 	if (is_proc_kcore) {
 		if (!vmcoreinfo_note) {
 			err = read_vmcoreinfo_fallback(&prog->reader,
-						       have_non_zero_phys_addr,
 						       &prog->vmcoreinfo);
 			if (err)
 				goto out_segments;
@@ -428,7 +475,6 @@ out_segments:
 	drgn_memory_reader_init(&prog->reader);
 	free(prog->file_segments);
 	prog->file_segments = NULL;
-	prog->num_file_segments = 0;
 out_elf:
 	elf_end(prog->core);
 	prog->core = NULL;
@@ -468,7 +514,6 @@ drgn_program_set_pid(struct drgn_program *prog, pid_t pid)
 	prog->file_segments[0].file_size = UINT64_MAX;
 	prog->file_segments[0].fd = prog->core_fd;
 	prog->file_segments[0].eio_is_fault = true;
-	prog->num_file_segments = 1;
 	err = drgn_program_add_memory_segment(prog, 0, UINT64_MAX,
 					      drgn_read_memory_file,
 					      prog->file_segments, false);
@@ -485,7 +530,6 @@ out_segments:
 	drgn_memory_reader_init(&prog->reader);
 	free(prog->file_segments);
 	prog->file_segments = NULL;
-	prog->num_file_segments = 0;
 out_fd:
 	close(prog->core_fd);
 	prog->core_fd = -1;
@@ -694,47 +738,71 @@ drgn_program_load_debug_info(struct drgn_program *prog, const char **paths,
 	return err;
 }
 
-struct drgn_error *drgn_program_cache_prstatus_entry(struct drgn_program *prog,
-						     char *data, size_t size)
+static uint32_t get_prstatus_pid(struct drgn_program *prog, const char *data,
+				 size_t size)
 {
-	struct drgn_prstatus_map_entry entry;
-	size_t pr_pid_offset;
 	uint32_t pr_pid;
-	bool bswap;
-
-	pr_pid_offset = drgn_program_is_64_bit(prog) ? 32 : 24;
-	bswap = (drgn_program_is_little_endian(prog) !=
-		 (__BYTE_ORDER__ == __ORDER_LITTLE_ENDIAN__));
-
-	if (size < pr_pid_offset + sizeof(pr_pid))
-		return NULL;
-
-	memcpy(&pr_pid, data + pr_pid_offset, sizeof(pr_pid));
-	if (bswap)
+	memcpy(&pr_pid, data + (drgn_program_is_64_bit(prog) ? 32 : 24),
+	       sizeof(pr_pid));
+	if (drgn_program_bswap(prog))
 		pr_pid = bswap_32(pr_pid);
-	if (!pr_pid)
-		return NULL;
+	return pr_pid;
+}
 
-	entry.key = pr_pid;
-	entry.value.str = data;
-	entry.value.len = size;
-	if (drgn_prstatus_map_insert(&prog->prstatus_cache, &entry,
-				     NULL) == -1) {
-		return &drgn_enomem;
+struct drgn_error *drgn_program_cache_prstatus_entry(struct drgn_program *prog,
+						     const char *data,
+						     size_t size)
+{
+	if (size < (drgn_program_is_64_bit(prog) ? 36 : 28)) {
+		return drgn_error_create(DRGN_ERROR_OTHER,
+					 "NT_PRSTATUS is truncated");
+	}
+	if (prog->flags & DRGN_PROGRAM_IS_LINUX_KERNEL) {
+		struct string *entry =
+			drgn_prstatus_vector_append_entry(&prog->prstatus_vector);
+		if (!entry)
+			return &drgn_enomem;
+		entry->str = data;
+		entry->len = size;
+	} else {
+		struct drgn_prstatus_map_entry entry = {
+			.key = get_prstatus_pid(prog, data, size),
+			.value = { data, size },
+		};
+		if (drgn_prstatus_map_insert(&prog->prstatus_map, &entry,
+					     NULL) == -1)
+			return &drgn_enomem;
 	}
 	return NULL;
 }
 
 static struct drgn_error *drgn_program_cache_prstatus(struct drgn_program *prog)
 {
+	struct drgn_error *err;
 	size_t phnum, i;
 
+	if (prog->prstatus_cached)
+		return NULL;
+
+	if (prog->flags & DRGN_PROGRAM_IS_LINUX_KERNEL)
+		drgn_prstatus_vector_init(&prog->prstatus_vector);
+	else
+		drgn_prstatus_map_init(&prog->prstatus_map);
+
 #ifdef WITH_LIBKDUMPFILE
-	if (prog->kdump_ctx)
-		return drgn_program_cache_prstatus_kdump(prog);
+	if (prog->kdump_ctx) {
+		err = drgn_program_cache_prstatus_kdump(prog);
+		goto out;
+	}
 #endif
-	if (elf_getphdrnum(prog->core, &phnum) != 0)
-		return drgn_error_libelf();
+	if (!prog->core) {
+		err = NULL;
+		goto out;
+	}
+	if (elf_getphdrnum(prog->core, &phnum) != 0) {
+		err = drgn_error_libelf();
+		goto out;
+	}
 	for (i = 0; i < phnum; i++) {
 		GElf_Phdr phdr_mem, *phdr;
 		Elf_Data *data;
@@ -743,23 +811,26 @@ static struct drgn_error *drgn_program_cache_prstatus(struct drgn_program *prog)
 		size_t name_offset, desc_offset;
 
 		phdr = gelf_getphdr(prog->core, i, &phdr_mem);
-		if (!phdr)
-			return drgn_error_libelf();
+		if (!phdr) {
+			err = drgn_error_libelf();
+			goto out;
+		}
 		if (phdr->p_type != PT_NOTE)
 			continue;
 
 		data = elf_getdata_rawchunk(prog->core, phdr->p_offset,
 					    phdr->p_filesz,
 					    note_header_type(phdr));
-		if (!data)
-			return drgn_error_libelf();
+		if (!data) {
+			err = drgn_error_libelf();
+			goto out;
+		}
 
 		offset = 0;
 		while (offset < data->d_size &&
 		       (offset = gelf_getnote(data, offset, &nhdr, &name_offset,
 					      &desc_offset))) {
 			const char *name;
-			struct drgn_error *err;
 
 			name = (char *)data->d_buf + name_offset;
 			if (strncmp(name, "CORE", nhdr.n_namesz) != 0 ||
@@ -770,26 +841,58 @@ static struct drgn_error *drgn_program_cache_prstatus(struct drgn_program *prog)
 								(char *)data->d_buf + desc_offset,
 								nhdr.n_descsz);
 			if (err)
-				return err;
+				goto out;
 		}
 	}
+
+	err = NULL;
+out:
+	if (err) {
+		if (prog->flags & DRGN_PROGRAM_IS_LINUX_KERNEL)
+			drgn_prstatus_vector_deinit(&prog->prstatus_vector);
+		else
+			drgn_prstatus_map_deinit(&prog->prstatus_map);
+	} else {
+		prog->prstatus_cached = true;
+	}
+	return err;
+}
+
+struct drgn_error *drgn_program_find_prstatus_by_cpu(struct drgn_program *prog,
+						     uint32_t cpu,
+						     struct string *ret,
+						     uint32_t *tid_ret)
+{
+	struct drgn_error *err;
+
+	assert(prog->flags & DRGN_PROGRAM_IS_LINUX_KERNEL);
+	err = drgn_program_cache_prstatus(prog);
+	if (err)
+		return err;
+
+	if (cpu < prog->prstatus_vector.size) {
+		*ret = prog->prstatus_vector.data[cpu];
+		*tid_ret = get_prstatus_pid(prog, ret->str, ret->len);
+	} else {
+		ret->str = NULL;
+		ret->len = 0;
+	}
 	return NULL;
 }
 
-struct drgn_error *drgn_program_find_prstatus(struct drgn_program *prog,
-					      uint32_t tid, struct string *ret)
+struct drgn_error *drgn_program_find_prstatus_by_tid(struct drgn_program *prog,
+						     uint32_t tid,
+						     struct string *ret)
 {
 	struct drgn_error *err;
 	struct drgn_prstatus_map_iterator it;
 
-	if (!prog->prstatus_cached) {
-		err = drgn_program_cache_prstatus(prog);
-		if (err)
-			return err;
-		prog->prstatus_cached = true;
-	}
+	assert(!(prog->flags & DRGN_PROGRAM_IS_LINUX_KERNEL));
+	err = drgn_program_cache_prstatus(prog);
+	if (err)
+		return err;
 
-	it = drgn_prstatus_map_search(&prog->prstatus_cache, &tid);
+	it = drgn_prstatus_map_search(&prog->prstatus_map, &tid);
 	if (!it.entry) {
 		ret->str = NULL;
 		ret->len = 0;
@@ -926,13 +1029,9 @@ drgn_program_read_c_string(struct drgn_program *prog, uint64_t address,
 			   bool physical, size_t max_size, char **ret)
 {
 	struct drgn_error *err;
-	struct char_vector str;
-
-	char_vector_init(&str);
+	struct char_vector str = VECTOR_INIT;
 	for (;;) {
-		char *c;
-
-		c = char_vector_append_entry(&str);
+		char *c = char_vector_append_entry(&str);
 		if (!c) {
 			char_vector_deinit(&str);
 			return &drgn_enomem;
@@ -957,6 +1056,73 @@ drgn_program_read_c_string(struct drgn_program *prog, uint64_t address,
 	return NULL;
 }
 
+LIBDRGN_PUBLIC struct drgn_error *
+drgn_program_read_u8(struct drgn_program *prog, uint64_t address, bool physical,
+		     uint8_t *ret)
+{
+	return drgn_memory_reader_read(&prog->reader, ret, address,
+				       sizeof(*ret), physical);
+}
+
+#define DEFINE_PROGRAM_READ_U(n)						\
+LIBDRGN_PUBLIC struct drgn_error *						\
+drgn_program_read_u##n(struct drgn_program *prog, uint64_t address,		\
+		       bool physical, uint##n##_t *ret)				\
+{										\
+	struct drgn_error *err;							\
+	uint##n##_t tmp;							\
+										\
+	if (!prog->has_platform) {						\
+		return drgn_error_create(DRGN_ERROR_INVALID_ARGUMENT,		\
+					 "program byte order is not known");	\
+	}									\
+	err = drgn_memory_reader_read(&prog->reader, &tmp, address,		\
+				      sizeof(tmp), physical);			\
+	if (err)								\
+		return err;							\
+	if (drgn_program_bswap(prog))						\
+		tmp = bswap_##n(tmp);						\
+	*ret = tmp;								\
+	return NULL;								\
+}
+
+DEFINE_PROGRAM_READ_U(16)
+DEFINE_PROGRAM_READ_U(32)
+DEFINE_PROGRAM_READ_U(64)
+#undef DEFINE_PROGRAM_READ_U
+
+LIBDRGN_PUBLIC struct drgn_error *
+drgn_program_read_word(struct drgn_program *prog, uint64_t address,
+		       bool physical, uint64_t *ret)
+{
+	struct drgn_error *err;
+
+	if (!prog->has_platform) {
+		return drgn_error_create(DRGN_ERROR_INVALID_ARGUMENT,
+					 "program word size is not known");
+	}
+	if (drgn_program_is_64_bit(prog)) {
+		uint64_t tmp;
+		err = drgn_memory_reader_read(&prog->reader, &tmp, address,
+					      sizeof(tmp), physical);
+		if (err)
+			return err;
+		if (drgn_program_bswap(prog))
+			tmp = bswap_64(tmp);
+		*ret = tmp;
+	} else {
+		uint32_t tmp;
+		err = drgn_memory_reader_read(&prog->reader, &tmp, address,
+					      sizeof(tmp), physical);
+		if (err)
+			return err;
+		if (drgn_program_bswap(prog))
+			tmp = bswap_32(tmp);
+		*ret = tmp;
+	}
+	return NULL;
+}
+
 LIBDRGN_PUBLIC struct drgn_error *
 drgn_program_find_type(struct drgn_program *prog, const char *name,
 		       const char *filename, struct drgn_qualified_type *ret)
