@@ -1,4 +1,4 @@
-// Copyright 2018-2020 - Omar Sandoval
+// Copyright (c) Facebook, Inc. and its affiliates.
 // SPDX-License-Identifier: GPL-3.0+
 
 /**
@@ -22,6 +22,7 @@
 #include "object_index.h"
 #include "platform.h"
 #include "type_index.h"
+#include "vector.h"
 
 /**
  * @ingroup Internals
@@ -47,10 +48,13 @@ struct vmcoreinfo {
 	 * is enabled.
 	 */
 	uint64_t kaslr_offset;
-	/* Whether 5-level paging was enabled. */
+	/** Kernel page table. */
+	uint64_t swapper_pg_dir;
+	/** Whether 5-level paging was enabled. */
 	bool pgtable_l5_enabled;
 };
 
+DEFINE_VECTOR_TYPE(drgn_prstatus_vector, struct string)
 DEFINE_HASH_MAP_TYPE(drgn_prstatus_map, uint32_t, struct string)
 
 struct drgn_dwarf_info_cache;
@@ -62,7 +66,6 @@ struct drgn_program {
 	struct drgn_type_index tindex;
 	struct drgn_object_index oindex;
 	struct drgn_memory_file_segment *file_segments;
-	size_t num_file_segments;
 	/* Default language of the program. */
 	const struct drgn_language *lang;
 	/*
@@ -73,6 +76,8 @@ struct drgn_program {
 	uint64_t page_offset;
 	/* Cached vmemmap. */
 	uint64_t vmemmap;
+	/* Cached THREAD_SIZE. */
+	uint64_t thread_size;
 #ifdef WITH_LIBKDUMPFILE
 	kdump_ctx_t *kdump_ctx;
 #endif
@@ -89,7 +94,16 @@ struct drgn_program {
 	  */
 	pid_t pid;
 	struct drgn_dwarf_info_cache *_dicache;
-	struct drgn_prstatus_map prstatus_cache;
+	union {
+		/*
+		 * For the Linux kernel, PRSTATUS notes indexed by CPU. See @ref
+		 * drgn_architecture_info::linux_kernel_set_initial_registers
+		 * for why we don't use the PID map.
+		 */
+		struct drgn_prstatus_vector prstatus_vector;
+		/* For userspace programs, PRSTATUS notes indexed by PID. */
+		struct drgn_prstatus_map prstatus_map;
+	};
 	/* See @ref drgn_object_stack_trace(). */
 	struct drgn_error *stack_trace_err;
 	/* See @ref drgn_object_stack_trace_next_thread(). */
@@ -100,7 +114,14 @@ struct drgn_program {
 	bool has_platform;
 	bool attached_dwfl_state;
 	bool prstatus_cached;
+	/*
+	 * Whether @ref drgn_program::pgtable_it is currently being used. Used
+	 * to prevent address translation from recursing.
+	 */
+	bool pgtable_it_in_use;
 
+	/* Page table iterator for linux_helper_read_vm(). */
+	struct pgtable_iterator *pgtable_it;
 	/* Cache for @ref linux_helper_task_state_to_char(). */
 	char *task_state_chars;
 	uint64_t task_report;
@@ -144,6 +165,16 @@ static inline bool drgn_program_is_little_endian(struct drgn_program *prog)
 	return prog->platform.flags & DRGN_PLATFORM_IS_LITTLE_ENDIAN;
 }
 
+/**
+ * Return whether a @ref drgn_program has a different endianness than the host
+ * system.
+ */
+static inline bool drgn_program_bswap(struct drgn_program *prog)
+{
+	return (drgn_program_is_little_endian(prog) !=
+		(__BYTE_ORDER__ == __ORDER_LITTLE_ENDIAN__));
+}
+
 static inline bool drgn_program_is_64_bit(struct drgn_program *prog)
 {
 	assert(prog->has_platform);
@@ -152,16 +183,31 @@ static inline bool drgn_program_is_64_bit(struct drgn_program *prog)
 
 struct drgn_error *drgn_program_get_dwfl(struct drgn_program *prog, Dwfl **ret);
 
+/**
+ * Find the @c NT_PRSTATUS note for the given CPU.
+ *
+ * This is only valid for the Linux kernel.
+ *
+ * @param[out] ret Returned note data. If not found, <tt>ret->str</tt> is set to
+ * @c NULL and <tt>ret->len</tt> is set to zero.
+ * @param[out] tid_ret Returned thread ID of note.
+ */
+struct drgn_error *drgn_program_find_prstatus_by_cpu(struct drgn_program *prog,
+						     uint32_t cpu,
+						     struct string *ret,
+						     uint32_t *tid_ret);
+
 /**
  * Find the @c NT_PRSTATUS note for the given thread ID.
  *
- * This assumes that <tt>prog->core</tt> is not @c NULL.
+ * This is only valid for userspace programs.
  *
  * @param[out] ret Returned note data. If not found, <tt>ret->str</tt> is set to
  * @c NULL and <tt>ret->len</tt> is set to zero.
  */
-struct drgn_error *drgn_program_find_prstatus(struct drgn_program *prog,
-					      uint32_t tid, struct string *ret);
+struct drgn_error *drgn_program_find_prstatus_by_tid(struct drgn_program *prog,
+						     uint32_t tid,
+						     struct string *ret);
 
 /**
  * Cache the @c NT_PRSTATUS note provided by @p data in @p prog.
@@ -170,7 +216,8 @@ struct drgn_error *drgn_program_find_prstatus(struct drgn_program *prog,
  * @param[in] size Size of data in note.
  */
 struct drgn_error *drgn_program_cache_prstatus_entry(struct drgn_program *prog,
-                                                     char *data, size_t size);
+                                                     const char *data,
+						     size_t size);
 
 /*
  * Like @ref drgn_program_find_symbol_by_address(), but @p ret is already
