@@ -1,4 +1,4 @@
-// Copyright 2018-2020 - Omar Sandoval
+// Copyright (c) Facebook, Inc. and its affiliates.
 // SPDX-License-Identifier: GPL-3.0+
 
 #ifndef DRGNPY_H
@@ -287,6 +287,8 @@ struct enum_arg {
 };
 int enum_converter(PyObject *o, void *p);
 
+PyObject *drgnpy_linux_helper_read_vm(PyObject *self, PyObject *args,
+				      PyObject *kwds);
 DrgnObject *drgnpy_linux_helper_radix_tree_lookup(PyObject *self,
 						  PyObject *args,
 						  PyObject *kwds);
@@ -300,6 +302,8 @@ DrgnObject *drgnpy_linux_helper_find_task(PyObject *self, PyObject *args,
 					  PyObject *kwds);
 PyObject *drgnpy_linux_helper_task_state_to_char(PyObject *self, PyObject *args,
 						 PyObject *kwds);
+PyObject *drgnpy_linux_helper_kaslr_offset(PyObject *self, PyObject *args,
+					   PyObject *kwds);
 PyObject *drgnpy_linux_helper_pgtable_l5_enabled(PyObject *self, PyObject *args,
 						 PyObject *kwds);
 
