@@ -1,3 +1,6 @@
+// Copyright (c) Facebook, Inc. and its affiliates.
+// SPDX-License-Identifier: GPL-3.0+
+
 #include <string.h>
 
 #include "internal.h"
@@ -648,6 +651,7 @@ static struct drgn_error *drgn_type_eq_impl(struct drgn_type *a,
 	(*depth)++;
 
 	if (drgn_type_kind(a) != drgn_type_kind(b) ||
+	    drgn_type_language(a) != drgn_type_language(b) ||
 	    drgn_type_is_complete(a) != drgn_type_is_complete(b))
 		goto out_false;
 
@@ -710,12 +714,9 @@ out:
 LIBDRGN_PUBLIC struct drgn_error *drgn_type_eq(struct drgn_type *a,
 					       struct drgn_type *b, bool *ret)
 {
-	struct drgn_error *err;
-	struct drgn_type_pair_set cache;
+	struct drgn_type_pair_set cache = HASH_TABLE_INIT;
 	int depth = 0;
-
-	drgn_type_pair_set_init(&cache);
-	err = drgn_type_eq_impl(a, b, &cache, &depth, ret);
+	struct drgn_error *err = drgn_type_eq_impl(a, b, &cache, &depth, ret);
 	drgn_type_pair_set_deinit(&cache);
 	return err;
 }
@@ -833,7 +834,7 @@ LIBDRGN_PUBLIC struct drgn_error *drgn_type_sizeof(struct drgn_type *type,
 		return drgn_error_create(DRGN_ERROR_TYPE,
 					 "cannot get size of function type");
 	}
-	DRGN_UNREACHABLE();
+	UNREACHABLE();
 }
 
 struct drgn_error *drgn_type_bit_size(struct drgn_type *type, uint64_t *ret)
@@ -879,7 +880,7 @@ enum drgn_object_kind drgn_type_object_kind(struct drgn_type *type)
 	case DRGN_TYPE_FUNCTION:
 		return DRGN_OBJECT_NONE;
 	}
-	DRGN_UNREACHABLE();
+	UNREACHABLE();
 }
 
 struct drgn_error *drgn_type_error(const char *format, struct drgn_type *type)
@@ -928,7 +929,7 @@ struct drgn_error *drgn_error_incomplete_type(const char *format,
 	case DRGN_TYPE_VOID:
 		return drgn_error_format(DRGN_ERROR_TYPE, format, "void");
 	default:
-		DRGN_UNREACHABLE();
+		UNREACHABLE();
 	}
 }
 
