@@ -1,4 +1,4 @@
-// Copyright 2019 - Omar Sandoval
+// Copyright (c) Facebook, Inc. and its affiliates.
 // SPDX-License-Identifier: GPL-3.0+
 
 /**
@@ -64,6 +64,8 @@ struct vector {
  * Initialize a @ref vector.
  *
  * The new vector is empty.
+ *
+ * @sa VECTOR_INIT
  */
 void vector_init(struct vector *vector);
 
@@ -230,6 +232,15 @@ static vector##_entry_type *vector##_pop(struct vector *vector)			\
 DEFINE_VECTOR_TYPE(vector, entry_type)		\
 DEFINE_VECTOR_FUNCTIONS(vector)
 
+/**
+ * Empty vector initializer.
+ *
+ * This can be used to initialize a vector when declaring it.
+ *
+ * @sa vector_init()
+ */
+#define VECTOR_INIT { NULL }
+
 /** @} */
 
 #endif /* DRGN_VECTOR_H */
