@@ -1,3 +1,6 @@
+# Copyright (c) Facebook, Inc. and its affiliates.
+# SPDX-License-Identifier: GPL-3.0+
+
 import math
 import operator
 import struct
@@ -39,16 +42,6 @@ from tests import (
 
 
 class TestInit(ObjectTestCase):
-    def test_reinit(self):
-        obj = Object(self.prog, "int", value=1)
-        self.assertEqual(obj.value_(), 1)
-        obj.__init__(self.prog, value=2)
-        self.assertEqual(obj.value_(), 2)
-        prog = mock_program()
-        self.assertRaisesRegex(
-            ValueError, "cannot change object program", obj.__init__, prog, value=3
-        )
-
     def test_type_stays_alive(self):
         obj = Object(self.prog, int_type("int", 4, True), value=0)
         self.assertEqual(obj.type_, int_type("int", 4, True))
@@ -77,6 +70,15 @@ class TestInit(ObjectTestCase):
             self.prog,
             "int",
         )
+        self.assertRaisesRegex(
+            ValueError,
+            "object cannot have address and value",
+            Object,
+            self.prog,
+            "int",
+            0,
+            address=0,
+        )
         self.assertRaisesRegex(
             ValueError,
             "object cannot have address and value",
@@ -370,6 +372,9 @@ class TestReference(ObjectTestCase):
 
 
 class TestValue(ObjectTestCase):
+    def test_positional(self):
+        self.assertEqual(Object(self.prog, "int", 1), Object(self.prog, "int", value=1))
+
     def test_signed(self):
         obj = Object(self.prog, "int", value=-4)
         self.assertIs(obj.prog_, self.prog)
@@ -1485,6 +1490,10 @@ class TestCOperators(ObjectTestCase):
         self.assertEqual(-Object(self.prog, "unsigned char", value=1), self.int(-1))
         self.assertEqual(-self.int(-1), self.int(1))
         self.assertEqual(-self.unsigned_int(1), self.unsigned_int(0xFFFFFFFF))
+        self.assertEqual(
+            -Object(self.prog, "long", value=-0x8000000000000000),
+            Object(self.prog, "long", value=-0x8000000000000000),
+        )
         self.assertEqual(-self.double(2.0), self.double(-2.0))
         self.assertRaisesRegex(
             TypeError,
